//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

#define FUNC_C
#include "func.h"
#include "signals.h"

void set_dev(unsigned char cno, char *val){ //     
	int r, tmp = atoi(val);
	if(tmp > 3 || tmp < 0) return;
	if(tmp == 0) r = send_cmd(ISSTEP, 1, cno);
	else r = send_cmd(ISNTSTEP, 1, cno);
	if(r > -1) warn(": %d\n", tmp);
};

void set_voltage(unsigned char cno, char *val){ //   
	int U=-1, tmp = atoi(val);
	if(tmp > 2 || tmp < 0) return;
	switch(tmp){
		case 2: if(send_cmd(SET24V, 1, cno)>-1) U=24; break;
		case 1: if(send_cmd(SET12V, 1, cno)>-1) U=12; break;
		default: if(send_cmd(SET09V, 1, cno)>-1) U=9; break;
	}
	if(U>0) warn(" : %d \n", U);	
}

void set_timer(unsigned char cno, char *val){
	float tmp = atof(val);
	int w2s = (int)(65536. - 125000. * tmp);
	if(w2s < 0 || w2s > 65533){
		warn("  0.524\n");
		return;
	}
	if(send_cmd(SET_TIMER, 1, cno)<0) return;
	sendword(w2s);
	warn("  \n");
}

void rotate_left(unsigned char cno, char *val){
	unsigned char dev = val[0] - '0';
	warn(" (); dev=%d cno=%d\n", dev, cno);
	switch_dev(cno, dev, STEP_LEFT, DPM1L, DPM2L, DPM1L2L);
}

void rotate_right(unsigned char cno, char *val){
	unsigned char dev = val[0] - '0';
	warn(" () dev=%d cno=%d\n", dev, cno);
	switch_dev(cno, dev, STEP_RIGHT, DPM1R, DPM2R, DPM1R2R);
};

void steps_(unsigned char cno, char dir, char *val){ //   N 
	int r, tmp = atoi(val);
	if(tmp > 0 && tmp < 65536){
		switch(dir){
			case LEFT: warn("  %d  ()\n", tmp); r = send_cmd(STEP_L_NUM, 1, cno);break;
			default: warn("  %d  ()\n", tmp); r = send_cmd(STEP_R_NUM, 1, cno); 
		}
		if(r > -1) sendword(tmp);
	}
	else err("    [1,65535]\n");
}

void corrector(unsigned char cno, const unsigned char dir, char *val){
	unsigned char cmd = 0;
	char wrds[20];
	int tmp = atoi(val);
	if(tmp > 0 && tmp < 65536){
		switch(dir){
			case 0: cmd = STEP_R_NUM; strcpy(wrds, "X ()"); break; // X+
			case 1: cmd = STEP_L_NUM; strcpy(wrds, "X ()");break; // X-
			case 2: cmd = STEP_R_NUM + 224; strcpy(wrds, "Y ()");break; // Y+ 
			case 3: cmd = STEP_L_NUM + 224; strcpy(wrds, "Y ()");break; // Y-
			default: return;
		}
		if(send_cmd(cmd, 0, cno) < 0) return;
		sendword(tmp);
		warn(": %s  %d \n", wrds, tmp);
	}
}

void stop_motor(unsigned char cno, char *val){
	unsigned char dev = val[0] - '0';
	warn("   dev=%d cno=%d\n", dev, cno);
	switch_dev(cno, dev, STEP_STOP, DPM1_STOP, DPM2_STOP, DPM12_STOP);
};

/*          
	 ( ): 1L2R, 1R2L,      , ...
*/

void terminals(unsigned char cno){ //  
	unsigned char term, num, i=0;
	do{
		term = read_tty();
		num = term & 0xE0;
		term &= 0x1F;
	}while(num != cno && i++ < 5);
	err(" %d", cno);
	if((term & 1) == 1){
		err("  1"); //       -
		}
	if(((term >> 1) & 1) == 0){
		err("  2");}
	if(((term >> 2) & 1) == 0){
		err("  3");}
	if(((term >> 3) & 1) == 0){
		err("  4");}
	if(((term >> 4) & 1) == 1){
		err("  5");}
}

void error(unsigned char erno, unsigned char cno){ //   RX 
	err("   %d", cno);
	switch(erno){	
		case STACK_OVERFLOW:
			err("!!!!!! :   RX !!!!!!"); //   
			break;
		case NO_STOP_BIT:
			err("!!!!!! :      !!!!!!"); 
			break;
	}
}


void switch_dev(unsigned char cno, unsigned char dev, unsigned char cmd_step, unsigned char cmd_drm1, unsigned char cmd_drm2, unsigned char cmd_drm12){ //       
	unsigned char cmd;
	switch(dev){
		case STEP: cmd = cmd_step; break;
		case DPM1: cmd = cmd_drm1; break;
		case DPM2: cmd = cmd_drm2; break;
		case DPM12: cmd = cmd_drm12; break;
		default: cmd = ERR_CMD;
	}
	if(cmd != ERR_CMD) send_cmd(cmd, 1, cno);
}

int goto_middle(unsigned char cno){
	unsigned char rr, st_h, st_l, cmd;
	int steps;
	double t;
	double t0 = dtime();
	cmd = TERMINAL_ | cno;
	if(send_cmd(STEP_R_NUM, 1,cno) < 0) return 0;
	sendword(65535); //    
	do{
		usleep(50000);
		rr = read_tty();
		t = dtime() - t0;
	}while(rr != cmd && t < 10.0);
	if(t > 10.0) return(0);
	terminals(cno); //     
	if(send_cmd(STEP_L_NUM, 1, cno) < 0) return 0;
	sendword(65535); //    
	t0 = dtime();
	do{
		usleep(50000);
		rr = read_tty();
		t = dtime() - t0;
	}while(rr != cmd && t < 10.0);
	if(t > 10.0) return(0);
	terminals(cno);
	if(send_cmd(TOTAL_STEPS, 1, cno) < 0) return 0; //  -   
	usleep(10000);
	read(comfd, &st_h, 1);
	usleep(10000);
	read(comfd, &st_l, 1);
	steps=(st_h<<8) | st_l;
	if(send_cmd(STEP_R_NUM, 1, cno) < 0) return 0; //     
	sendword(steps/2);
	warn("    \n");
	return 1;
}

int init_motor(unsigned char cno){ //  
	//unsigned int mask = 0;
	unsigned int mask = 37262;// 0b1001000110001110; //   ,
	/*         :
	  |TS|  A | stop_mask ||TF|  B | mode_mask |
	H/| 7| 6 5| 4 3 2 1 0 ||7 | 6 5| 4 3 2 1 0 |/L
	TS - terminal_stop_motor (1-  )
	TF - terminal_flag (1- )
	mode_mask -     
	stop_mask - 1-     1
	AB-stop_mask2 - 1-   2 
	*/
	if(send_cmd(ISSTEP, 1, cno) < 0) return 0;
	if(send_cmd(SET_TIMER, 1, cno) < 0) return 0;
	sendword(65411); // 0.1 sec
	if(send_cmd(TERM_METHOD, 1, cno) < 0) return 0;
	sendword(mask);
	return 1;
}

