//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

#include "queues.h"
#include "keys.h"
mqd_t mq_cmd = -1, mq_out = -1;
ssize_t mq_size = 0;
char *in_buf = NULL, *out_buf = NULL;
void mk_queues(){
	attr.mq_maxmsg = 1024; //  1024   
	attr.mq_flags = O_NONBLOCK;
	attr.mq_msgsize = 128;
	mq_cmd = mq_open(CMD_QUEUE, O_RDONLY|O_CREAT|O_NONBLOCK, 00666, &attr);
	if(mq_cmd < 0){perror("Error opening cmd queue"); exit(1);}
	mq_out = mq_open(OUT_QUEUE, O_WRONLY|O_CREAT|O_NONBLOCK, 00666, &attr);
	if(mq_out < 0){perror("Error opening out queue"); exit(1);}
	if(mq_getattr(mq_cmd, &attr) < 0){perror("Error getting attributes");exit(1);}
	if(mq_getattr(mq_out, &attr) < 0){perror("Error getting attributes");exit(1);}
	mq_size = attr.mq_msgsize;
	if((in_buf = (char*)calloc(mq_size, 1)) == NULL){perror("alloc");exit(1);}
	if((out_buf= (char*)calloc(mq_size, 1)) == NULL){perror("alloc");exit(1);}
	mq_size--; //     
}

char* get_qs(char* buf, int l){
	char *m, *qs = NULL;
	if((m = getenv("REQUEST_METHOD")) && strcasecmp(m, "POST") == 0)
		qs = fgets(buf, l, stdin);
	else if( (qs = getenv("QUERY_STRING")) )
		qs = strncpy(buf, qs, l);
	if(qs && strlen(qs) < 1) qs = NULL;
	return qs;
}

int get_qs_param(char *param, char *meaning, int l){
	char *tok, *val, *par, *str;
	int stat = 0;
	str = strdup(in_buf);
//fprintf(stderr, "get_qs_param: %s\n", str);
	tok = strtok(str, "&,;| \n");
	do{
		if((val = strchr(tok, '=')) == NULL) continue;
		*val++ = '\0';
		par = tok;
		if(strcasecmp(par, param)==0){
			stat = 1;
			strncpy(meaning, val, l);
			break;
		}
	}while((tok = strtok(NULL, "&,;| \n"))!=NULL);
	free(str);
	return stat;
}

int read_queue(unsigned char *cno, unsigned char *cmd, char *value){
	ssize_t cur, n;
	unsigned prio;
	char *par;
	int tmp;
	mq_getattr(mq_cmd, &attr);
	cur = attr.mq_curmsgs;
	if(cur == 0) return (-1);
	n = mq_receive(mq_cmd, in_buf, attr.mq_msgsize, &prio);
	if(n < 1) return (-1);
	par = (char*)calloc(mq_size+1, 1);
	in_buf[n] = 0;
	if(get_qs_param("port", par, mq_size)){
		*cmd = 0; //  
		strncpy(value, par, mq_size);
		free(par); return 1;
	}
	if(get_qs_param("button", par, mq_size) == 0){ free(par); return (0);}
	*cmd = par[0];
	if(par[0] == KEY_QUIT){ free(par); return 1;}
	if(get_qs_param("ctrlr", par, mq_size) == 0){ free(par); return (-1);}
	tmp = atoi(par);
	if(tmp <0 || tmp > 7){ free(par); return(-1);}
	*cno = (unsigned char)tmp << 5;
	if(get_qs_param("data", par, mq_size) == 0) *value = 0;
	else strncpy(value, par, mq_size);
	free(par);
	return 1;
}

void rm_queues(){
	if(in_buf) free(in_buf);
	if(out_buf) free(out_buf);
	if(mq_cmd > 0){
		mq_close(mq_cmd);
		if(mq_unlink(CMD_QUEUE) != 0) perror("Error unlink cmd queue");
	}
	if(mq_out > 0){
		mq_close(mq_out);
		if(mq_unlink(OUT_QUEUE) != 0) perror("Error unlink cmd queue");
	}
}
